@@ -36,22 +36,23 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
 
 public class RssSparkShuffleUtilsTest {
-  
+
   private static final String EXPECTED_EXCEPTION_MESSAGE = "Exception should be thrown";
-  
+
   @Test
   public void testAssignmentTags() {
     SparkConf conf = new SparkConf();
 
     /**
-     * Case 1: don't set the tag implicitly and will return the {@code Constants.SHUFFLE_SERVER_VERSION}
-      */
+     * Case 1: don't set the tag implicitly and will return the {@code
+     * Constants.SHUFFLE_SERVER_VERSION}
+     */
     Set<String> tags = RssSparkShuffleUtils.getAssignmentTags(conf);
     assertEquals(Constants.SHUFFLE_SERVER_VERSION, tags.iterator().next());
 
     /**
-     * Case 2: set the multiple tags implicitly and will return the {@code Constants.SHUFFLE_SERVER_VERSION}
-     * and configured tags.
+     * Case 2: set the multiple tags implicitly and will return the {@code
+     * Constants.SHUFFLE_SERVER_VERSION} and configured tags.
      */
     conf.set(RssSparkConfig.RSS_CLIENT_ASSIGNMENT_TAGS.key(), " a,b");
     tags = RssSparkShuffleUtils.getAssignmentTags(conf);
@@ -79,7 +80,9 @@ public class RssSparkShuffleUtilsTest {
     assertEquals("org.apache.hadoop.odfs.HdfsOdfs", conf1.get("fs.AbstractFileSystem.hdfs.impl"));
 
     conf.set(RssSparkConfig.RSS_OZONE_FS_HDFS_IMPL.key(), "expect_odfs_impl");
-    conf.set(RssSparkConfig.RSS_OZONE_FS_ABSTRACT_FILE_SYSTEM_HDFS_IMPL.key(), "expect_odfs_abstract_impl");
+    conf.set(
+        RssSparkConfig.RSS_OZONE_FS_ABSTRACT_FILE_SYSTEM_HDFS_IMPL.key(),
+        "expect_odfs_abstract_impl");
     conf1 = RssSparkShuffleUtils.newHadoopConfiguration(conf);
     assertEquals("expect_odfs_impl", conf1.get("fs.hdfs.impl"));
     assertEquals("expect_odfs_abstract_impl", conf1.get("fs.AbstractFileSystem.hdfs.impl"));
@@ -95,57 +98,81 @@ public class RssSparkShuffleUtilsTest {
 
     clientConf.put(RssClientConfig.RSS_REMOTE_STORAGE_PATH, remoteStoragePath);
     clientConf.put(RssClientConfig.RSS_CLIENT_TYPE, RssClientConfig.RSS_CLIENT_TYPE_DEFAULT_VALUE);
-    clientConf.put(RssClientConfig.RSS_CLIENT_RETRY_MAX,
+    clientConf.put(
+        RssClientConfig.RSS_CLIENT_RETRY_MAX,
         Integer.toString(RssClientConfig.RSS_CLIENT_RETRY_MAX_DEFAULT_VALUE));
-    clientConf.put(RssClientConfig.RSS_CLIENT_RETRY_INTERVAL_MAX,
+    clientConf.put(
+        RssClientConfig.RSS_CLIENT_RETRY_INTERVAL_MAX,
         Long.toString(RssClientConfig.RSS_CLIENT_RETRY_INTERVAL_MAX_DEFAULT_VALUE));
-    clientConf.put(RssClientConfig.RSS_DATA_REPLICA,
+    clientConf.put(
+        RssClientConfig.RSS_DATA_REPLICA,
         Integer.toString(RssClientConfig.RSS_DATA_REPLICA_DEFAULT_VALUE));
-    clientConf.put(RssClientConfig.RSS_DATA_REPLICA_WRITE,
+    clientConf.put(
+        RssClientConfig.RSS_DATA_REPLICA_WRITE,
         Integer.toString(RssClientConfig.RSS_DATA_REPLICA_WRITE_DEFAULT_VALUE));
-    clientConf.put(RssClientConfig.RSS_DATA_REPLICA_READ,
+    clientConf.put(
+        RssClientConfig.RSS_DATA_REPLICA_READ,
         Integer.toString(RssClientConfig.RSS_DATA_REPLICA_READ_DEFAULT_VALUE));
-    clientConf.put(RssClientConfig.RSS_HEARTBEAT_INTERVAL,
+    clientConf.put(
+        RssClientConfig.RSS_HEARTBEAT_INTERVAL,
         Long.toString(RssClientConfig.RSS_HEARTBEAT_INTERVAL_DEFAULT_VALUE));
     clientConf.put(RssClientConfig.RSS_STORAGE_TYPE, StorageType.MEMORY_LOCALFILE_HDFS.name());
-    clientConf.put(RssClientConfig.RSS_CLIENT_SEND_CHECK_INTERVAL_MS,
+    clientConf.put(
+        RssClientConfig.RSS_CLIENT_SEND_CHECK_INTERVAL_MS,
         Long.toString(RssClientConfig.RSS_CLIENT_SEND_CHECK_INTERVAL_MS_DEFAULT_VALUE));
-    clientConf.put(RssClientConfig.RSS_CLIENT_SEND_CHECK_TIMEOUT_MS,
+    clientConf.put(
+        RssClientConfig.RSS_CLIENT_SEND_CHECK_TIMEOUT_MS,
         Long.toString(RssClientConfig.RSS_CLIENT_SEND_CHECK_TIMEOUT_MS_DEFAULT_VALUE));
-    clientConf.put(RssClientConfig.RSS_PARTITION_NUM_PER_RANGE,
+    clientConf.put(
+        RssClientConfig.RSS_PARTITION_NUM_PER_RANGE,
         Integer.toString(RssClientConfig.RSS_PARTITION_NUM_PER_RANGE_DEFAULT_VALUE));
-    clientConf.put(RssClientConfig.RSS_INDEX_READ_LIMIT,
+    clientConf.put(
+        RssClientConfig.RSS_INDEX_READ_LIMIT,
         Integer.toString(RssClientConfig.RSS_INDEX_READ_LIMIT_DEFAULT_VALUE));
-    clientConf.put(RssClientConfig.RSS_CLIENT_READ_BUFFER_SIZE,
+    clientConf.put(
+        RssClientConfig.RSS_CLIENT_READ_BUFFER_SIZE,
         RssClientConfig.RSS_CLIENT_READ_BUFFER_SIZE_DEFAULT_VALUE);
     clientConf.put(mockKey, mockValue);
 
     RssSparkShuffleUtils.applyDynamicClientConf(conf, clientConf);
     assertEquals(remoteStoragePath, conf.get(RssSparkConfig.RSS_REMOTE_STORAGE_PATH.key()));
-    assertEquals(RssClientConfig.RSS_CLIENT_TYPE_DEFAULT_VALUE,
+    assertEquals(
+        RssClientConfig.RSS_CLIENT_TYPE_DEFAULT_VALUE,
         conf.get(RssSparkConfig.RSS_CLIENT_TYPE.key()));
-    assertEquals(Integer.toString(RssClientConfig.RSS_CLIENT_RETRY_MAX_DEFAULT_VALUE),
+    assertEquals(
+        Integer.toString(RssClientConfig.RSS_CLIENT_RETRY_MAX_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_CLIENT_RETRY_MAX.key()));
-    assertEquals(Long.toString(RssClientConfig.RSS_CLIENT_RETRY_INTERVAL_MAX_DEFAULT_VALUE),
+    assertEquals(
+        Long.toString(RssClientConfig.RSS_CLIENT_RETRY_INTERVAL_MAX_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_CLIENT_RETRY_INTERVAL_MAX.key()));
-    assertEquals(Integer.toString(RssClientConfig.RSS_DATA_REPLICA_DEFAULT_VALUE),
+    assertEquals(
+        Integer.toString(RssClientConfig.RSS_DATA_REPLICA_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_DATA_REPLICA.key()));
-    assertEquals(Integer.toString(RssClientConfig.RSS_DATA_REPLICA_WRITE_DEFAULT_VALUE),
+    assertEquals(
+        Integer.toString(RssClientConfig.RSS_DATA_REPLICA_WRITE_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_DATA_REPLICA_WRITE.key()));
-    assertEquals(Integer.toString(RssClientConfig.RSS_DATA_REPLICA_READ_DEFAULT_VALUE),
+    assertEquals(
+        Integer.toString(RssClientConfig.RSS_DATA_REPLICA_READ_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_DATA_REPLICA_READ.key()));
-    assertEquals(Long.toString(RssClientConfig.RSS_HEARTBEAT_INTERVAL_DEFAULT_VALUE),
+    assertEquals(
+        Long.toString(RssClientConfig.RSS_HEARTBEAT_INTERVAL_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_HEARTBEAT_INTERVAL.key()));
-    assertEquals(StorageType.MEMORY_LOCALFILE_HDFS.name(), conf.get(RssSparkConfig.RSS_STORAGE_TYPE.key()));
-    assertEquals(Long.toString(RssClientConfig.RSS_CLIENT_SEND_CHECK_INTERVAL_MS_DEFAULT_VALUE),
+    assertEquals(
+        StorageType.MEMORY_LOCALFILE_HDFS.name(), conf.get(RssSparkConfig.RSS_STORAGE_TYPE.key()));
+    assertEquals(
+        Long.toString(RssClientConfig.RSS_CLIENT_SEND_CHECK_INTERVAL_MS_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_CLIENT_SEND_CHECK_INTERVAL_MS.key()));
-    assertEquals(Long.toString(RssClientConfig.RSS_CLIENT_SEND_CHECK_TIMEOUT_MS_DEFAULT_VALUE),
+    assertEquals(
+        Long.toString(RssClientConfig.RSS_CLIENT_SEND_CHECK_TIMEOUT_MS_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_CLIENT_SEND_CHECK_TIMEOUT_MS.key()));
-    assertEquals(Integer.toString(RssClientConfig.RSS_PARTITION_NUM_PER_RANGE_DEFAULT_VALUE),
+    assertEquals(
+        Integer.toString(RssClientConfig.RSS_PARTITION_NUM_PER_RANGE_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_PARTITION_NUM_PER_RANGE.key()));
-    assertEquals(Integer.toString(RssClientConfig.RSS_INDEX_READ_LIMIT_DEFAULT_VALUE),
+    assertEquals(
+        Integer.toString(RssClientConfig.RSS_INDEX_READ_LIMIT_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_INDEX_READ_LIMIT.key()));
-    assertEquals(RssClientConfig.RSS_CLIENT_READ_BUFFER_SIZE_DEFAULT_VALUE,
+    assertEquals(
+        RssClientConfig.RSS_CLIENT_READ_BUFFER_SIZE_DEFAULT_VALUE,
         conf.get(RssSparkConfig.RSS_CLIENT_READ_BUFFER_SIZE.key()));
     assertEquals(mockValue, conf.get(mockKey));
 
@@ -161,7 +188,8 @@ public class RssSparkShuffleUtilsTest {
     assertEquals(StorageType.MEMORY_HDFS.name(), conf.get(RssSparkConfig.RSS_STORAGE_TYPE.key()));
     // won't be overwrite
     assertEquals(mockValue, conf.get(mockKey));
-    assertEquals(Integer.toString(RssClientConfig.RSS_CLIENT_RETRY_MAX_DEFAULT_VALUE),
+    assertEquals(
+        Integer.toString(RssClientConfig.RSS_CLIENT_RETRY_MAX_DEFAULT_VALUE),
         conf.get(RssSparkConfig.RSS_CLIENT_RETRY_MAX.key()));
   }
 
@@ -240,5 +268,4 @@ public class RssSparkShuffleUtilsTest {
     sparkConf.set(RssSparkConfig.RSS_CLIENT_SEND_CHECK_TIMEOUT_MS, 5000L);
     RssSparkShuffleUtils.validateRssClientConf(sparkConf);
   }
-
 }
