@@ -171,69 +171,76 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
         serverToBlocks.entrySet()) {
       CompletableFuture<Boolean> future =
           CompletableFuture.supplyAsync(
-              () -> {
-                if (needCancelRequest.get()) {
-                  LOG.info("The upstream task has been failed. Abort this data send.");
-                  return true;
-                }
-                ShuffleServerInfo ssi = entry.getKey();
-                try {
-                  Map<Integer, Map<Integer, List<ShuffleBlockInfo>>> shuffleIdToBlocks =
-                      entry.getValue();
-                  // todo: compact unnecessary blocks that reach replicaWrite
-                  RssSendShuffleDataRequest request =
-                      new RssSendShuffleDataRequest(
-                          appId, retryMax, retryIntervalMax, shuffleIdToBlocks);
-                  long s = System.currentTimeMillis();
-                  RssSendShuffleDataResponse response =
-                      getShuffleServerClient(ssi).sendShuffleData(request);
+                  () -> {
+                    if (needCancelRequest.get()) {
+                      LOG.info("The upstream task has been failed. Abort this data send.");
+                      return true;
+                    }
+                    ShuffleServerInfo ssi = entry.getKey();
+                    try {
+                      Map<Integer, Map<Integer, List<ShuffleBlockInfo>>> shuffleIdToBlocks =
+                          entry.getValue();
+                      // todo: compact unnecessary blocks that reach replicaWrite
+                      RssSendShuffleDataRequest request =
+                          new RssSendShuffleDataRequest(
+                              appId, retryMax, retryIntervalMax, shuffleIdToBlocks);
+                      long s = System.currentTimeMillis();
+                      RssSendShuffleDataResponse response =
+                          getShuffleServerClient(ssi).sendShuffleData(request);
 
-                  String logMsg =
-                      String.format(
-                          "ShuffleWriteClientImpl sendShuffleData with %s blocks to %s cost: %s(ms)",
-                          serverToBlockIds.get(ssi).size(),
-                          ssi.getId(),
-                          System.currentTimeMillis() - s);
+                      String logMsg =
+                          String.format(
+                              "ShuffleWriteClientImpl sendShuffleData with %s blocks to %s cost: %s(ms)",
+                              serverToBlockIds.get(ssi).size(),
+                              ssi.getId(),
+                              System.currentTimeMillis() - s);
 
-                  if (response.getStatusCode() == StatusCode.SUCCESS) {
-                    // mark a replica of block that has been sent
-                    serverToBlockIds
-                        .get(ssi)
-                        .forEach(
-                            blockId -> blockIdsSendSuccessTracker.get(blockId).incrementAndGet());
-                    if (defectiveServers != null) {
-                      defectiveServers.remove(ssi);
+                      if (response.getStatusCode() == StatusCode.SUCCESS) {
+                        // mark a replica of block that has been sent
+                        serverToBlockIds
+                            .get(ssi)
+                            .forEach(
+                                blockId ->
+                                    blockIdsSendSuccessTracker.get(blockId).incrementAndGet());
+                        if (defectiveServers != null) {
+                          defectiveServers.remove(ssi);
+                        }
+                        if (LOG.isDebugEnabled()) {
+                          LOG.debug("{} successfully.", logMsg);
+                        }
+                      } else {
+                        recordFailedBlocks(
+                            failedBlockSendTracker, serverToBlocks, ssi, response.getStatusCode());
+                        if (defectiveServers != null) {
+                          defectiveServers.add(ssi);
+                        }
+                        LOG.warn(
+                            "{}, it failed wth statusCode[{}]", logMsg, response.getStatusCode());
+                        return false;
+                      }
+                    } catch (Exception e) {
+                      recordFailedBlocks(
+                          failedBlockSendTracker, serverToBlocks, ssi, StatusCode.INTERNAL_ERROR);
+                      if (defectiveServers != null) {
+                        defectiveServers.add(ssi);
+                      }
+                      LOG.warn(
+                          "Send: "
+                              + serverToBlockIds.get(ssi).size()
+                              + " blocks to ["
+                              + ssi.getId()
+                              + "] failed.",
+                          e);
+                      return false;
                     }
-                    if (LOG.isDebugEnabled()) {
-                      LOG.debug("{} successfully.", logMsg);
-                    }
-                  } else {
-                    recordFailedBlocks(
-                        failedBlockSendTracker, serverToBlocks, ssi, response.getStatusCode());
-                    if (defectiveServers != null) {
-                      defectiveServers.add(ssi);
-                    }
-                    LOG.warn("{}, it failed wth statusCode[{}]", logMsg, response.getStatusCode());
+                    return true;
+                  },
+                  dataTransferPool)
+              .exceptionally(
+                  ex -> {
+                    LOG.error("Unexpected exceptions occurred while sending shuffle data", ex);
                     return false;
-                  }
-                } catch (Exception e) {
-                  recordFailedBlocks(
-                      failedBlockSendTracker, serverToBlocks, ssi, StatusCode.INTERNAL_ERROR);
-                  if (defectiveServers != null) {
-                    defectiveServers.add(ssi);
-                  }
-                  LOG.warn(
-                      "Send: "
-                          + serverToBlockIds.get(ssi).size()
-                          + " blocks to ["
-                          + ssi.getId()
-                          + "] failed.",
-                      e);
-                  return false;
-                }
-                return true;
-              },
-              dataTransferPool);
+                  });
       futures.add(future);
     }
 
