@@ -31,7 +31,11 @@ public class ShufflePartitionedData {
 
   @Override
   public String toString() {
-    return "ShufflePartitionedData{partitionId=" + partitionId + ", blockList=" + Arrays.toString(blockList) + '}';
+    return "ShufflePartitionedData{partitionId="
+        + partitionId
+        + ", blockList="
+        + Arrays.toString(blockList)
+        + '}';
   }
 
   public int getPartitionId() {
@@ -44,7 +48,7 @@ public class ShufflePartitionedData {
 
   public ShufflePartitionedBlock[] getBlockList() {
     if (blockList == null) {
-      return new ShufflePartitionedBlock[]{};
+      return new ShufflePartitionedBlock[] {};
     }
     return blockList;
   }
@@ -59,5 +63,4 @@ public class ShufflePartitionedData {
     }
     return size;
   }
-
 }
