@@ -21,21 +21,49 @@ import java.nio.ByteBuffer;
 
 import net.jpountz.lz4.LZ4Factory;
 
+import org.apache.uniffle.common.exception.RssException;
+
 public class Lz4Codec extends Codec {
 
   private LZ4Factory lz4Factory;
 
+  private static class LazyHolder {
+    static final Lz4Codec INSTANCE = new Lz4Codec();
+  }
+
+  public static Lz4Codec getInstance() {
+    return LazyHolder.INSTANCE;
+  }
+
   public Lz4Codec() {
     this.lz4Factory = LZ4Factory.fastestInstance();
   }
 
   @Override
   public void decompress(ByteBuffer src, int uncompressedLen, ByteBuffer dest, int destOffset) {
-    lz4Factory.fastDecompressor().decompress(src, src.position(), dest, destOffset, uncompressedLen);
+    lz4Factory
+        .fastDecompressor()
+        .decompress(src, src.position(), dest, destOffset, uncompressedLen);
   }
 
   @Override
   public byte[] compress(byte[] src) {
     return lz4Factory.fastCompressor().compress(src);
   }
+
+  @Override
+  public int compress(ByteBuffer src, ByteBuffer dest) {
+    try {
+      int destOff = dest.position();
+      lz4Factory.fastCompressor().compress(src.duplicate(), dest);
+      return dest.position() - destOff;
+    } catch (Exception e) {
+      throw new RssException("Failed to compress by Lz4", e);
+    }
+  }
+
+  @Override
+  public int maxCompressedLength(int sourceLength) {
+    return lz4Factory.fastCompressor().maxCompressedLength(sourceLength);
+  }
 }
