@@ -29,11 +29,11 @@ import static org.junit.jupiter.api.Assertions.assertThrows;
 
 public class UnitConverterTest {
 
-  private static final long PB = (long)ByteUnit.PiB.toBytes(1L);
-  private static final long TB = (long)ByteUnit.TiB.toBytes(1L);
-  private static final long GB = (long)ByteUnit.GiB.toBytes(1L);
-  private static final long MB = (long)ByteUnit.MiB.toBytes(1L);
-  private static final long KB = (long)ByteUnit.KiB.toBytes(1L);
+  private static final long PB = (long) ByteUnit.PiB.toBytes(1L);
+  private static final long TB = (long) ByteUnit.TiB.toBytes(1L);
+  private static final long GB = (long) ByteUnit.GiB.toBytes(1L);
+  private static final long MB = (long) ByteUnit.MiB.toBytes(1L);
+  private static final long KB = (long) ByteUnit.KiB.toBytes(1L);
 
   private static Stream<Arguments> byteStringArgs() {
     return Stream.of(
@@ -42,36 +42,29 @@ public class UnitConverterTest {
         Arguments.arguments(10 * PB, "10pB", ByteUnit.BYTE),
         Arguments.arguments(10 * PB, "10p", ByteUnit.BYTE),
         Arguments.arguments(10 * PB, "10P", ByteUnit.BYTE),
-
         Arguments.arguments(10 * TB, "10TB", ByteUnit.BYTE),
         Arguments.arguments(10 * TB, "10tb", ByteUnit.BYTE),
         Arguments.arguments(10 * TB, "10tB", ByteUnit.BYTE),
         Arguments.arguments(10 * TB, "10T", ByteUnit.BYTE),
         Arguments.arguments(10 * TB, "10t", ByteUnit.BYTE),
-
         Arguments.arguments(10 * GB, "10GB", ByteUnit.BYTE),
         Arguments.arguments(10 * GB, "10gb", ByteUnit.BYTE),
         Arguments.arguments(10 * GB, "10gB", ByteUnit.BYTE),
-
         Arguments.arguments(10 * MB, "10MB", ByteUnit.BYTE),
         Arguments.arguments(10 * MB, "10mb", ByteUnit.BYTE),
         Arguments.arguments(10 * MB, "10mB", ByteUnit.BYTE),
         Arguments.arguments(10 * MB, "10M", ByteUnit.BYTE),
         Arguments.arguments(10 * MB, "10m", ByteUnit.BYTE),
-
         Arguments.arguments(10 * KB, "10KB", ByteUnit.BYTE),
         Arguments.arguments(10 * KB, "10kb", ByteUnit.BYTE),
         Arguments.arguments(10 * KB, "10Kb", ByteUnit.BYTE),
         Arguments.arguments(10 * KB, "10K", ByteUnit.BYTE),
         Arguments.arguments(10 * KB, "10k", ByteUnit.BYTE),
-
         Arguments.arguments(1111L, "1111", ByteUnit.BYTE),
-
         Arguments.arguments(null, "1/2", ByteUnit.BYTE),
         Arguments.arguments(null, "10f", ByteUnit.BYTE),
         Arguments.arguments(null, "f91", ByteUnit.BYTE),
-        Arguments.arguments(null, "1.0", ByteUnit.BYTE)
-    );
+        Arguments.arguments(null, "1.0", ByteUnit.BYTE));
   }
 
   @ParameterizedTest
@@ -100,11 +93,9 @@ public class UnitConverterTest {
         Arguments.arguments(3 * MIN, "3min", TimeUnit.MICROSECONDS),
         Arguments.arguments(3 * HOUR, "3h", TimeUnit.MICROSECONDS),
         Arguments.arguments(3 * DAY, "3d", TimeUnit.MICROSECONDS),
-
         Arguments.arguments(5L, "5", TimeUnit.MILLISECONDS),
         Arguments.arguments(5L, "5", TimeUnit.SECONDS),
         Arguments.arguments(5L, "5", TimeUnit.MINUTES),
-
         Arguments.arguments(null, "3ns", TimeUnit.MICROSECONDS),
         Arguments.arguments(null, "3sec", TimeUnit.MICROSECONDS),
         Arguments.arguments(null, "1.5h", TimeUnit.MICROSECONDS),
@@ -112,8 +103,7 @@ public class UnitConverterTest {
         Arguments.arguments(null, "3days", TimeUnit.MICROSECONDS),
         Arguments.arguments(null, "3w", TimeUnit.MICROSECONDS),
         Arguments.arguments(null, "3weeks", TimeUnit.MICROSECONDS),
-        Arguments.arguments(null, "foo", TimeUnit.MICROSECONDS)
-    );
+        Arguments.arguments(null, "foo", TimeUnit.MICROSECONDS));
   }
 
   @ParameterizedTest
