@@ -253,6 +253,28 @@ public class SimpleClusterManager implements ClusterManager {
     return availableNodes;
   }
 
+  @Override
+  public List<ServerNode> getServerList(Set<String> requiredTags, Set<String> faultyServerIds) {
+    List<ServerNode> availableNodes = Lists.newArrayList();
+    for (ServerNode node : servers.values()) {
+      if (!ServerStatus.ACTIVE.equals(node.getStatus())) {
+        continue;
+      }
+      if (isNodeAvailable(requiredTags, faultyServerIds, node)) {
+        availableNodes.add(node);
+      }
+    }
+    return availableNodes;
+  }
+
+  private boolean isNodeAvailable(
+      Set<String> requiredTags, Set<String> faultyServerIds, ServerNode node) {
+    if (faultyServerIds != null && faultyServerIds.contains(node.getId())) {
+      return false;
+    }
+    return !excludeNodes.contains(node.getId()) && node.getTags().containsAll(requiredTags);
+  }
+
   @Override
   public List<ServerNode> getLostServerList() {
     return Lists.newArrayList(lostNodes);
@@ -263,6 +285,7 @@ public class SimpleClusterManager implements ClusterManager {
     return Lists.newArrayList(unhealthyNodes);
   }
 
+  @Override
   public Set<String> getExcludeNodes() {
     return excludeNodes;
   }
