@@ -20,6 +20,7 @@ package org.apache.uniffle.test;
 import java.io.File;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
@@ -30,19 +31,20 @@ import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
 import org.roaringbitmap.longlong.Roaring64NavigableMap;
 
+import org.apache.uniffle.client.factory.ShuffleClientFactory;
 import org.apache.uniffle.client.factory.ShuffleServerClientFactory;
 import org.apache.uniffle.client.impl.ShuffleReadClientImpl;
 import org.apache.uniffle.client.impl.ShuffleWriteClientImpl;
 import org.apache.uniffle.client.impl.grpc.ShuffleServerGrpcClient;
 import org.apache.uniffle.client.response.CompressedShuffleBlock;
 import org.apache.uniffle.client.response.SendShuffleDataResult;
-import org.apache.uniffle.client.util.DefaultIdHelper;
 import org.apache.uniffle.common.ClientType;
 import org.apache.uniffle.common.PartitionRange;
 import org.apache.uniffle.common.RemoteStorageInfo;
 import org.apache.uniffle.common.ShuffleBlockInfo;
 import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleServerInfo;
+import org.apache.uniffle.common.rpc.ServerType;
 import org.apache.uniffle.common.util.RssUtils;
 import org.apache.uniffle.coordinator.CoordinatorConf;
 import org.apache.uniffle.coordinator.CoordinatorServer;
@@ -71,15 +73,26 @@ public class QuorumTest extends ShuffleReadWriteBase {
   private static ShuffleServerInfo fakedShuffleServerInfo4;
   private MockedShuffleWriteClientImpl shuffleWriteClientImpl;
 
+  private ShuffleClientFactory.ReadClientBuilder baseReadBuilder() {
+    return ShuffleClientFactory.newReadBuilder()
+        .clientType(ClientType.GRPC)
+        .storageType(StorageType.MEMORY_LOCALFILE.name())
+        .shuffleId(0)
+        .partitionId(0)
+        .indexReadLimit(100)
+        .partitionNumPerRange(1)
+        .partitionNum(10)
+        .readBufferSize(1000);
+  }
+
   public static MockedShuffleServer createServer(int id, File tmpDir) throws Exception {
-    ShuffleServerConf shuffleServerConf = getShuffleServerConf();
+    ShuffleServerConf shuffleServerConf = getShuffleServerConf(ServerType.GRPC);
     shuffleServerConf.setLong("rss.server.app.expired.withoutHeartbeat", 8000);
     shuffleServerConf.setLong("rss.server.heartbeat.interval", 5000);
     File dataDir1 = new File(tmpDir, id + "_1");
     File dataDir2 = new File(tmpDir, id + "_2");
     String basePath = dataDir1.getAbsolutePath() + "," + dataDir2.getAbsolutePath();
     shuffleServerConf.setString("rss.storage.type", StorageType.MEMORY_LOCALFILE.name());
-    shuffleServerConf.setInteger("rss.rpc.server.port", SHUFFLE_SERVER_PORT + id);
     shuffleServerConf.setInteger("rss.jetty.http.port", 19081 + id * 100);
     shuffleServerConf.setString("rss.storage.basePath", basePath);
     return new MockedShuffleServer(shuffleServerConf);
@@ -90,53 +103,73 @@ public class QuorumTest extends ShuffleReadWriteBase {
     CoordinatorConf coordinatorConf = getCoordinatorConf();
     createCoordinatorServer(coordinatorConf);
 
-    ShuffleServerConf shuffleServerConf = getShuffleServerConf();
+    ShuffleServerConf shuffleServerConf = getShuffleServerConf(ServerType.GRPC);
     shuffleServerConf.setLong("rss.server.app.expired.withoutHeartbeat", 8000);
 
-    shuffleServers.add(createServer(0, tmpDir));
-    shuffleServers.add(createServer(1, tmpDir));
-    shuffleServers.add(createServer(2, tmpDir));
-    shuffleServers.add(createServer(3, tmpDir));
-    shuffleServers.add(createServer(4, tmpDir));
+    grpcShuffleServers.add(createServer(0, tmpDir));
+    grpcShuffleServers.add(createServer(1, tmpDir));
+    grpcShuffleServers.add(createServer(2, tmpDir));
+    grpcShuffleServers.add(createServer(3, tmpDir));
+    grpcShuffleServers.add(createServer(4, tmpDir));
 
     shuffleServerInfo0 =
         new ShuffleServerInfo(
-            "127.0.0.1-20001", shuffleServers.get(0).getIp(), SHUFFLE_SERVER_PORT + 0);
+            String.format("127.0.0.1-%s", grpcShuffleServers.get(0).getGrpcPort()),
+            grpcShuffleServers.get(0).getIp(),
+            grpcShuffleServers.get(0).getGrpcPort());
     shuffleServerInfo1 =
         new ShuffleServerInfo(
-            "127.0.0.1-20002", shuffleServers.get(1).getIp(), SHUFFLE_SERVER_PORT + 1);
+            String.format("127.0.0.1-%s", grpcShuffleServers.get(1).getGrpcPort()),
+            grpcShuffleServers.get(1).getIp(),
+            grpcShuffleServers.get(1).getGrpcPort());
     shuffleServerInfo2 =
         new ShuffleServerInfo(
-            "127.0.0.1-20003", shuffleServers.get(2).getIp(), SHUFFLE_SERVER_PORT + 2);
+            String.format("127.0.0.1-%s", grpcShuffleServers.get(2).getGrpcPort()),
+            grpcShuffleServers.get(2).getIp(),
+            grpcShuffleServers.get(2).getGrpcPort());
     shuffleServerInfo3 =
         new ShuffleServerInfo(
-            "127.0.0.1-20004", shuffleServers.get(3).getIp(), SHUFFLE_SERVER_PORT + 3);
+            String.format("127.0.0.1-%s", grpcShuffleServers.get(3).getGrpcPort()),
+            grpcShuffleServers.get(3).getIp(),
+            grpcShuffleServers.get(3).getGrpcPort());
     shuffleServerInfo4 =
         new ShuffleServerInfo(
-            "127.0.0.1-20005", shuffleServers.get(4).getIp(), SHUFFLE_SERVER_PORT + 4);
+            String.format("127.0.0.1-%s", grpcShuffleServers.get(4).getGrpcPort()),
+            grpcShuffleServers.get(4).getIp(),
+            grpcShuffleServers.get(4).getGrpcPort());
     for (CoordinatorServer coordinator : coordinators) {
       coordinator.start();
     }
-    for (ShuffleServer shuffleServer : shuffleServers) {
+    for (ShuffleServer shuffleServer : grpcShuffleServers) {
       shuffleServer.start();
     }
 
     // simulator of failed servers
     fakedShuffleServerInfo0 =
         new ShuffleServerInfo(
-            "127.0.0.1-20001", shuffleServers.get(0).getIp(), SHUFFLE_SERVER_PORT + 100);
+            "127.0.0.1-20001",
+            grpcShuffleServers.get(0).getIp(),
+            grpcShuffleServers.get(0).getGrpcPort() + 100);
     fakedShuffleServerInfo1 =
         new ShuffleServerInfo(
-            "127.0.0.1-20002", shuffleServers.get(1).getIp(), SHUFFLE_SERVER_PORT + 200);
+            "127.0.0.1-20002",
+            grpcShuffleServers.get(1).getIp(),
+            grpcShuffleServers.get(0).getGrpcPort() + 200);
     fakedShuffleServerInfo2 =
         new ShuffleServerInfo(
-            "127.0.0.1-20003", shuffleServers.get(2).getIp(), SHUFFLE_SERVER_PORT + 300);
+            "127.0.0.1-20003",
+            grpcShuffleServers.get(2).getIp(),
+            grpcShuffleServers.get(0).getGrpcPort() + 300);
     fakedShuffleServerInfo3 =
         new ShuffleServerInfo(
-            "127.0.0.1-20004", shuffleServers.get(2).getIp(), SHUFFLE_SERVER_PORT + 400);
+            "127.0.0.1-20004",
+            grpcShuffleServers.get(2).getIp(),
+            grpcShuffleServers.get(0).getGrpcPort() + 400);
     fakedShuffleServerInfo4 =
         new ShuffleServerInfo(
-            "127.0.0.1-20005", shuffleServers.get(2).getIp(), SHUFFLE_SERVER_PORT + 500);
+            "127.0.0.1-20005",
+            grpcShuffleServers.get(2).getIp(),
+            grpcShuffleServers.get(0).getGrpcPort() + 500);
 
     // spark.rss.data.replica=3
     // spark.rss.data.replica.write=2
@@ -157,23 +190,12 @@ public class QuorumTest extends ShuffleReadWriteBase {
     Thread.sleep(2000);
   }
 
-  public static void cleanCluster() throws Exception {
-    for (CoordinatorServer coordinator : coordinators) {
-      coordinator.stopServer();
-    }
-    for (ShuffleServer shuffleServer : shuffleServers) {
-      shuffleServer.stopServer();
-    }
-    shuffleServers = Lists.newArrayList();
-    coordinators = Lists.newArrayList();
-  }
-
   @AfterEach
   public void cleanEnv() throws Exception {
     if (shuffleWriteClientImpl != null) {
       shuffleWriteClientImpl.close();
     }
-    cleanCluster();
+    shutdownServers();
     // we need recovery `rpcTime`, or some unit tests may fail
     ((ShuffleServerGrpcClient)
             ShuffleServerClientFactory.getInstance()
@@ -243,22 +265,15 @@ public class QuorumTest extends ShuffleReadWriteBase {
     assertEquals(blockIdBitmap, succBlockIdBitmap);
 
     Roaring64NavigableMap taskIdBitmap = Roaring64NavigableMap.bitmapOf(0);
+
     ShuffleReadClientImpl readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, fakedShuffleServerInfo2),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(
+                Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, fakedShuffleServerInfo2))
+            .build();
     // The data should be read
     validateResult(readClient, expectedData);
 
@@ -300,32 +315,8 @@ public class QuorumTest extends ShuffleReadWriteBase {
   }
 
   static class MockedShuffleWriteClientImpl extends ShuffleWriteClientImpl {
-    MockedShuffleWriteClientImpl(
-        String clientType,
-        int retryMax,
-        long retryIntervalMax,
-        int heartBeatThreadNum,
-        int replica,
-        int replicaWrite,
-        int replicaRead,
-        boolean replicaSkipEnabled,
-        int dataTranferPoolSize,
-        int dataCommitPoolSize,
-        int unregisterThreadPoolSize,
-        int unregisterRequestTimeSec) {
-      super(
-          clientType,
-          retryMax,
-          retryIntervalMax,
-          heartBeatThreadNum,
-          replica,
-          replicaWrite,
-          replicaRead,
-          replicaSkipEnabled,
-          dataTranferPoolSize,
-          dataCommitPoolSize,
-          unregisterThreadPoolSize,
-          unregisterRequestTimeSec);
+    MockedShuffleWriteClientImpl(ShuffleClientFactory.WriteClientBuilder builder) {
+      super(builder);
     }
 
     public SendShuffleDataResult sendShuffleData(
@@ -339,18 +330,19 @@ public class QuorumTest extends ShuffleReadWriteBase {
 
     shuffleWriteClientImpl =
         new MockedShuffleWriteClientImpl(
-            ClientType.GRPC.name(),
-            3,
-            1000,
-            1,
-            replica,
-            replicaWrite,
-            replicaRead,
-            replicaSkip,
-            1,
-            1,
-            10,
-            10);
+            ShuffleClientFactory.newWriteBuilder()
+                .clientType(ClientType.GRPC.name())
+                .retryMax(3)
+                .retryIntervalMax(1000)
+                .heartBeatThreadNum(1)
+                .replica(replica)
+                .replicaWrite(replicaWrite)
+                .replicaRead(replicaRead)
+                .replicaSkipEnabled(replicaSkip)
+                .dataTransferPoolSize(1)
+                .dataCommitPoolSize(1)
+                .unregisterThreadPoolSize(10)
+                .unregisterRequestTimeSec(10));
 
     List<ShuffleServerInfo> allServers =
         Lists.newArrayList(
@@ -379,16 +371,16 @@ public class QuorumTest extends ShuffleReadWriteBase {
     Roaring64NavigableMap blockIdBitmap = Roaring64NavigableMap.bitmapOf();
 
     // only 1 server is timout, the block sending should success
-    enableTimeout((MockedShuffleServer) shuffleServers.get(2), 500);
+    enableTimeout((MockedShuffleServer) grpcShuffleServers.get(2), 500);
 
     // report result should success
-    Map<Integer, List<Long>> partitionToBlockIds = Maps.newHashMap();
-    partitionToBlockIds.put(0, Lists.newArrayList(blockIdBitmap.stream().iterator()));
-    Map<Integer, List<ShuffleServerInfo>> partitionToServers = Maps.newHashMap();
-    partitionToServers.put(
-        0, Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2));
-    shuffleWriteClientImpl.reportShuffleResult(
-        partitionToServers, testAppId, 0, 0L, partitionToBlockIds, 1);
+    Map<Integer, Set<Long>> partitionToBlockIds = Maps.newHashMap();
+    partitionToBlockIds.put(0, Sets.newHashSet(blockIdBitmap.stream().iterator()));
+    Map<ShuffleServerInfo, Map<Integer, Set<Long>>> serverToPartitionToBlockIds = Maps.newHashMap();
+    serverToPartitionToBlockIds.put(shuffleServerInfo0, partitionToBlockIds);
+    serverToPartitionToBlockIds.put(shuffleServerInfo1, partitionToBlockIds);
+    serverToPartitionToBlockIds.put(shuffleServerInfo2, partitionToBlockIds);
+    shuffleWriteClientImpl.reportShuffleResult(serverToPartitionToBlockIds, testAppId, 0, 0L, 1);
     Roaring64NavigableMap report =
         shuffleWriteClientImpl.getShuffleResult(
             "GRPC",
@@ -420,21 +412,13 @@ public class QuorumTest extends ShuffleReadWriteBase {
 
     Roaring64NavigableMap taskIdBitmap = Roaring64NavigableMap.bitmapOf(0);
     ShuffleReadClientImpl readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(
+                Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2))
+            .build();
     validateResult(readClient, expectedData);
   }
 
@@ -447,9 +431,8 @@ public class QuorumTest extends ShuffleReadWriteBase {
     Roaring64NavigableMap blockIdBitmap = Roaring64NavigableMap.bitmapOf();
     Roaring64NavigableMap taskIdBitmap = Roaring64NavigableMap.bitmapOf(0);
     // When 2 servers are timeout, the block sending should fail
-    enableTimeout((MockedShuffleServer) shuffleServers.get(1), 500);
-    enableTimeout((MockedShuffleServer) shuffleServers.get(2), 500);
-
+    enableTimeout((MockedShuffleServer) grpcShuffleServers.get(1), 500);
+    enableTimeout((MockedShuffleServer) grpcShuffleServers.get(2), 500);
     List<ShuffleBlockInfo> blocks =
         createShuffleBlockList(
             0,
@@ -469,14 +452,14 @@ public class QuorumTest extends ShuffleReadWriteBase {
     assertEquals(0, result.getSuccessBlockIds().size());
 
     // report result should fail
-    Map<Integer, List<Long>> partitionToBlockIds = Maps.newHashMap();
-    Map<Integer, List<ShuffleServerInfo>> partitionToServers = Maps.newHashMap();
-    partitionToBlockIds.put(0, Lists.newArrayList(blockIdBitmap.stream().iterator()));
-    partitionToServers.put(
-        0, Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2));
+    Map<Integer, Set<Long>> partitionToBlockIds = Maps.newHashMap();
+    partitionToBlockIds.put(0, Sets.newHashSet(blockIdBitmap.stream().iterator()));
+    Map<ShuffleServerInfo, Map<Integer, Set<Long>>> serverToPartitionToBlockIds = Maps.newHashMap();
+    serverToPartitionToBlockIds.put(shuffleServerInfo0, partitionToBlockIds);
+    serverToPartitionToBlockIds.put(shuffleServerInfo1, partitionToBlockIds);
+    serverToPartitionToBlockIds.put(shuffleServerInfo2, partitionToBlockIds);
     try {
-      shuffleWriteClientImpl.reportShuffleResult(
-          partitionToServers, testAppId, 0, 0L, partitionToBlockIds, 1);
+      shuffleWriteClientImpl.reportShuffleResult(serverToPartitionToBlockIds, testAppId, 0, 0L, 1);
       fail(EXPECTED_EXCEPTION_MESSAGE);
     } catch (Exception e) {
       assertTrue(e.getMessage().startsWith("Quorum check of report shuffle result is failed"));
@@ -499,12 +482,12 @@ public class QuorumTest extends ShuffleReadWriteBase {
   public void case3() throws Exception {
     String testAppId = "case3";
     registerShuffleServer(testAppId, 3, 2, 2, true);
-    disableTimeout((MockedShuffleServer) shuffleServers.get(0));
-    disableTimeout((MockedShuffleServer) shuffleServers.get(1));
-    disableTimeout((MockedShuffleServer) shuffleServers.get(2));
+    disableTimeout((MockedShuffleServer) grpcShuffleServers.get(0));
+    disableTimeout((MockedShuffleServer) grpcShuffleServers.get(1));
+    disableTimeout((MockedShuffleServer) grpcShuffleServers.get(2));
 
     // When 1 server is timeout and 1 server is failed after sending, the block sending should fail
-    enableTimeout((MockedShuffleServer) shuffleServers.get(2), 500);
+    enableTimeout((MockedShuffleServer) grpcShuffleServers.get(2), 500);
 
     Map<Long, byte[]> expectedData = Maps.newHashMap();
     Roaring64NavigableMap blockIdBitmap = Roaring64NavigableMap.bitmapOf();
@@ -530,13 +513,14 @@ public class QuorumTest extends ShuffleReadWriteBase {
     assertEquals(blockIdBitmap, succBlockIdBitmap);
     assertEquals(0, failedBlockIdBitmap.getLongCardinality());
 
-    Map<Integer, List<Long>> partitionToBlockIds = Maps.newHashMap();
-    partitionToBlockIds.put(0, Lists.newArrayList(blockIdBitmap.stream().iterator()));
-    Map<Integer, List<ShuffleServerInfo>> partitionToServers = Maps.newHashMap();
-    partitionToServers.put(
-        0, Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2));
-    shuffleWriteClientImpl.reportShuffleResult(
-        partitionToServers, testAppId, 0, 0L, partitionToBlockIds, 1);
+    Map<Integer, Set<Long>> partitionToBlockIds = Maps.newHashMap();
+    partitionToBlockIds.put(0, Sets.newHashSet(blockIdBitmap.stream().iterator()));
+    Map<ShuffleServerInfo, Map<Integer, Set<Long>>> serverToPartitionToBlockIds = Maps.newHashMap();
+    serverToPartitionToBlockIds.put(shuffleServerInfo0, partitionToBlockIds);
+    serverToPartitionToBlockIds.put(shuffleServerInfo1, partitionToBlockIds);
+    serverToPartitionToBlockIds.put(shuffleServerInfo2, partitionToBlockIds);
+
+    shuffleWriteClientImpl.reportShuffleResult(serverToPartitionToBlockIds, testAppId, 0, 0L, 1);
 
     Roaring64NavigableMap report =
         shuffleWriteClientImpl.getShuffleResult(
@@ -548,7 +532,7 @@ public class QuorumTest extends ShuffleReadWriteBase {
     assertEquals(report, blockIdBitmap);
 
     // let this server be failed, the reading will be also be failed
-    shuffleServers.get(1).stopServer();
+    grpcShuffleServers.get(1).stopServer();
     try {
       report =
           shuffleWriteClientImpl.getShuffleResult(
@@ -563,7 +547,7 @@ public class QuorumTest extends ShuffleReadWriteBase {
     }
 
     // When the timeout of one server is recovered, the block sending should success
-    disableTimeout((MockedShuffleServer) shuffleServers.get(2));
+    disableTimeout((MockedShuffleServer) grpcShuffleServers.get(2));
     report =
         shuffleWriteClientImpl.getShuffleResult(
             "GRPC",
@@ -579,7 +563,7 @@ public class QuorumTest extends ShuffleReadWriteBase {
     String testAppId = "case4";
     registerShuffleServer(testAppId, 3, 2, 2, true);
     // when 1 server is timeout, the sending multiple blocks should success
-    enableTimeout((MockedShuffleServer) shuffleServers.get(2), 500);
+    enableTimeout((MockedShuffleServer) grpcShuffleServers.get(2), 500);
 
     Map<Long, byte[]> expectedData = Maps.newHashMap();
     Roaring64NavigableMap blockIdBitmap = Roaring64NavigableMap.bitmapOf();
@@ -601,21 +585,13 @@ public class QuorumTest extends ShuffleReadWriteBase {
     }
 
     ShuffleReadClientImpl readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(
+                Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2))
+            .build();
     validateResult(readClient, expectedData);
   }
 
@@ -639,13 +615,13 @@ public class QuorumTest extends ShuffleReadWriteBase {
             Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2));
 
     // report result should success
-    Map<Integer, List<Long>> partitionToBlockIds = Maps.newHashMap();
-    partitionToBlockIds.put(0, Lists.newArrayList(blockIdBitmap.stream().iterator()));
-    Map<Integer, List<ShuffleServerInfo>> partitionToServers = Maps.newHashMap();
-    partitionToServers.put(
-        0, Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2));
-    shuffleWriteClientImpl.reportShuffleResult(
-        partitionToServers, testAppId, 0, 0L, partitionToBlockIds, 1);
+    Map<Integer, Set<Long>> partitionToBlockIds = Maps.newHashMap();
+    partitionToBlockIds.put(0, Sets.newHashSet(blockIdBitmap.stream().iterator()));
+    Map<ShuffleServerInfo, Map<Integer, Set<Long>>> serverToPartitionToBlockIds = Maps.newHashMap();
+    serverToPartitionToBlockIds.put(shuffleServerInfo0, partitionToBlockIds);
+    serverToPartitionToBlockIds.put(shuffleServerInfo1, partitionToBlockIds);
+    serverToPartitionToBlockIds.put(shuffleServerInfo2, partitionToBlockIds);
+    shuffleWriteClientImpl.reportShuffleResult(serverToPartitionToBlockIds, testAppId, 0, 0L, 1);
     Roaring64NavigableMap report =
         shuffleWriteClientImpl.getShuffleResult(
             "GRPC",
@@ -665,9 +641,9 @@ public class QuorumTest extends ShuffleReadWriteBase {
     assertEquals(blockIdBitmap, succBlockIdBitmap);
 
     // when one server is restarted, getShuffleResult should success
-    shuffleServers.get(1).stopServer();
-    shuffleServers.set(1, createServer(1, tmpDir));
-    shuffleServers.get(1).start();
+    grpcShuffleServers.get(1).stopServer();
+    grpcShuffleServers.set(1, createServer(1, tmpDir));
+    grpcShuffleServers.get(1).start();
     report =
         shuffleWriteClientImpl.getShuffleResult(
             "GRPC",
@@ -678,9 +654,9 @@ public class QuorumTest extends ShuffleReadWriteBase {
     assertEquals(report, blockIdBitmap);
 
     // when two servers are restarted, getShuffleResult should fail
-    shuffleServers.get(2).stopServer();
-    shuffleServers.set(2, createServer(2, tmpDir));
-    shuffleServers.get(2).start();
+    grpcShuffleServers.get(2).stopServer();
+    grpcShuffleServers.set(2, createServer(2, tmpDir));
+    grpcShuffleServers.get(2).start();
     try {
       report =
           shuffleWriteClientImpl.getShuffleResult(
@@ -736,26 +712,24 @@ public class QuorumTest extends ShuffleReadWriteBase {
             Lists.newArrayList(shuffleServerInfo2, shuffleServerInfo3, shuffleServerInfo4));
 
     // server 0,1,2 are ok, server 3,4 are timout
-    enableTimeout((MockedShuffleServer) shuffleServers.get(3), 500);
-    enableTimeout((MockedShuffleServer) shuffleServers.get(4), 500);
+    enableTimeout((MockedShuffleServer) grpcShuffleServers.get(3), 500);
+    enableTimeout((MockedShuffleServer) grpcShuffleServers.get(4), 500);
 
-    Map<Integer, List<Long>> partitionToBlockIds = Maps.newHashMap();
-    partitionToBlockIds.put(0, Lists.newArrayList(blockIdBitmap0.stream().iterator()));
-    partitionToBlockIds.put(1, Lists.newArrayList(blockIdBitmap1.stream().iterator()));
-    partitionToBlockIds.put(2, Lists.newArrayList(blockIdBitmap2.stream().iterator()));
-
-    Map<Integer, List<ShuffleServerInfo>> partitionToServers = Maps.newHashMap();
-    partitionToServers.put(
-        0, Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2));
-    partitionToServers.put(
-        1, Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2));
-    partitionToServers.put(
-        2, Lists.newArrayList(shuffleServerInfo2, shuffleServerInfo3, shuffleServerInfo4));
+    Map<Integer, Set<Long>> partitionToBlockIds = Maps.newHashMap();
+    partitionToBlockIds.put(0, Sets.newHashSet(blockIdBitmap0.stream().iterator()));
+    partitionToBlockIds.put(1, Sets.newHashSet(blockIdBitmap1.stream().iterator()));
+    Map<ShuffleServerInfo, Map<Integer, Set<Long>>> serverToPartitionToBlockIds = Maps.newHashMap();
+    serverToPartitionToBlockIds.put(shuffleServerInfo0, partitionToBlockIds);
+    serverToPartitionToBlockIds.put(shuffleServerInfo1, partitionToBlockIds);
+    serverToPartitionToBlockIds.put(shuffleServerInfo2, partitionToBlockIds);
 
+    Map<Integer, Set<Long>> partitionToBlockIds2 = Maps.newHashMap();
+    partitionToBlockIds2.put(2, Sets.newHashSet(blockIdBitmap2.stream().iterator()));
+    serverToPartitionToBlockIds.put(shuffleServerInfo3, partitionToBlockIds2);
+    serverToPartitionToBlockIds.put(shuffleServerInfo4, partitionToBlockIds2);
     // report result should fail because partition2 is failed to report server 3,4
     try {
-      shuffleWriteClientImpl.reportShuffleResult(
-          partitionToServers, testAppId, 0, 0L, partitionToBlockIds, 1);
+      shuffleWriteClientImpl.reportShuffleResult(serverToPartitionToBlockIds, testAppId, 0, 0L, 1);
       fail(EXPECTED_EXCEPTION_MESSAGE);
     } catch (Exception e) {
       assertTrue(e.getMessage().startsWith("Quorum check of report shuffle result is failed"));
@@ -790,59 +764,33 @@ public class QuorumTest extends ShuffleReadWriteBase {
 
     // we cannot read any blocks from server 2
     ShuffleReadClientImpl readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo2),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo2))
+            .build();
     assertTrue(readClient.readShuffleBlockData() == null);
 
     // we can read blocks from server 0,1
     readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1))
+            .build();
     validateResult(readClient, expectedData);
 
     // we can also read blocks from server 0,1,2
     readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(
+                Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2))
+            .build();
     validateResult(readClient, expectedData);
   }
 
@@ -876,59 +824,33 @@ public class QuorumTest extends ShuffleReadWriteBase {
 
     // we cannot read any blocks from server 1
     ShuffleReadClientImpl readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo1),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo1))
+            .build();
     assertTrue(readClient.readShuffleBlockData() == null);
 
     // we can read blocks from server 2, which is sent in to secondary round
     readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo2),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo2))
+            .build();
     validateResult(readClient, expectedData);
 
     // we can read blocks from server 0,1,2
     readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo0, fakedShuffleServerInfo1, shuffleServerInfo2),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(
+                Lists.newArrayList(shuffleServerInfo0, fakedShuffleServerInfo1, shuffleServerInfo2))
+            .build();
     validateResult(readClient, expectedData);
   }
 
@@ -966,40 +888,23 @@ public class QuorumTest extends ShuffleReadWriteBase {
 
     // we cannot read any blocks from server 3, 4
     ShuffleReadClientImpl readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo3, shuffleServerInfo4),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo3, shuffleServerInfo4))
+            .build();
     assertTrue(readClient.readShuffleBlockData() == null);
 
     // we can also read blocks from server 0,1,2
     readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(
+                Lists.newArrayList(shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2))
+            .build();
     validateResult(readClient, expectedData);
   }
 
@@ -1038,40 +943,22 @@ public class QuorumTest extends ShuffleReadWriteBase {
 
     // we cannot read any blocks from server 1 due to failures
     ShuffleReadClientImpl readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo1),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo1))
+            .build();
     assertTrue(readClient.readShuffleBlockData() == null);
 
     // we can also read blocks from server 3,4
     readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo3, shuffleServerInfo4),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo3, shuffleServerInfo4))
+            .build();
     validateResult(readClient, expectedData);
   }
 
@@ -1109,41 +996,24 @@ public class QuorumTest extends ShuffleReadWriteBase {
 
     // we cannot read any blocks from server 4 because the secondary round is skipped
     ShuffleReadClientImpl readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo4),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo4))
+            .build();
     assertTrue(readClient.readShuffleBlockData() == null);
 
     // we can read blocks from server 0,1,2,3
     readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(
-                shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2, shuffleServerInfo3),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(
+                Lists.newArrayList(
+                    shuffleServerInfo0, shuffleServerInfo1, shuffleServerInfo2, shuffleServerInfo3))
+            .build();
     validateResult(readClient, expectedData);
   }
 
@@ -1175,59 +1045,32 @@ public class QuorumTest extends ShuffleReadWriteBase {
 
     // we can read blocks from server 0
     ShuffleReadClientImpl readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo0),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo0))
+            .build();
     validateResult(readClient, expectedData);
 
     // we can also read blocks from server 1
     readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo1),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo1))
+            .build();
     validateResult(readClient, expectedData);
 
     // we can also read blocks from server 2
     readClient =
-        new ShuffleReadClientImpl(
-            StorageType.MEMORY_LOCALFILE.name(),
-            testAppId,
-            0,
-            0,
-            100,
-            1,
-            10,
-            1000,
-            "",
-            blockIdBitmap,
-            taskIdBitmap,
-            Lists.newArrayList(shuffleServerInfo2),
-            null,
-            new DefaultIdHelper());
+        baseReadBuilder()
+            .appId(testAppId)
+            .blockIdBitmap(blockIdBitmap)
+            .taskIdBitmap(taskIdBitmap)
+            .shuffleServerInfoList(Lists.newArrayList(shuffleServerInfo2))
+            .build();
     validateResult(readClient, expectedData);
   }
 
