@@ -90,6 +90,12 @@ public class CoordinatorGrpcClient extends GrpcClient implements CoordinatorClie
   public CoordinatorGrpcClient(String host, int port, int maxRetryAttempts, boolean usePlaintext) {
     super(host, port, maxRetryAttempts, usePlaintext);
     blockingStub = CoordinatorServerGrpc.newBlockingStub(channel);
+    LOG.info(
+        "Created CoordinatorGrpcClient, host:{}, port:{}, maxRetryAttempts:{}, usePlaintext:{}",
+        host,
+        port,
+        maxRetryAttempts,
+        usePlaintext);
   }
 
   public CoordinatorGrpcClient(ManagedChannel channel) {
@@ -145,7 +151,7 @@ public class CoordinatorGrpcClient extends GrpcClient implements CoordinatorClie
       response = blockingStub.withDeadlineAfter(timeout, TimeUnit.MILLISECONDS).heartbeat(request);
       status = response.getStatus();
     } catch (StatusRuntimeException e) {
-      LOG.error(e.getMessage());
+      LOG.error("Failed to doSendHeartBeat, request: {}", request, e);
       status = RssProtos.StatusCode.TIMEOUT;
     } catch (Exception e) {
       LOG.error(e.getMessage());
@@ -171,8 +177,8 @@ public class CoordinatorGrpcClient extends GrpcClient implements CoordinatorClie
       int dataReplica,
       Set<String> requiredTags,
       int assignmentShuffleServerNumber,
-      int estimateTaskConcurrency) {
-
+      int estimateTaskConcurrency,
+      Set<String> faultyServerIds) {
     RssProtos.GetShuffleServerRequest getServerRequest =
         RssProtos.GetShuffleServerRequest.newBuilder()
             .setApplicationId(appId)
@@ -183,6 +189,7 @@ public class CoordinatorGrpcClient extends GrpcClient implements CoordinatorClie
             .addAllRequireTags(requiredTags)
             .setAssignmentShuffleServerNumber(assignmentShuffleServerNumber)
             .setEstimateTaskConcurrency(estimateTaskConcurrency)
+            .addAllFaultyServerIds(faultyServerIds)
             .build();
 
     return blockingStub.getShuffleAssignments(getServerRequest);
@@ -275,7 +282,8 @@ public class CoordinatorGrpcClient extends GrpcClient implements CoordinatorClie
             request.getDataReplica(),
             request.getRequiredTags(),
             request.getAssignmentShuffleServerNumber(),
-            request.getEstimateTaskConcurrency());
+            request.getEstimateTaskConcurrency(),
+            request.getFaultyServerIds());
 
     RssGetShuffleAssignmentsResponse response;
     RssProtos.StatusCode statusCode = rpcResponse.getStatus();
