@@ -17,6 +17,7 @@
 
 package org.apache.uniffle.client.impl.grpc;
 
+import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicLong;
@@ -113,11 +114,13 @@ public class ShuffleServerGrpcNettyClient extends ShuffleServerGrpcClient {
       int shuffleId = stb.getKey();
       int size = 0;
       int blockNum = 0;
+      List<Integer> partitionIds = new ArrayList<>();
       for (Map.Entry<Integer, List<ShuffleBlockInfo>> ptb : stb.getValue().entrySet()) {
         for (ShuffleBlockInfo sbi : ptb.getValue()) {
           size += sbi.getSize();
           blockNum++;
         }
+        partitionIds.add(ptb.getKey());
       }
 
       SendShuffleDataRequest sendShuffleDataRequest =
@@ -137,6 +140,8 @@ public class ShuffleServerGrpcNettyClient extends ShuffleServerGrpcClient {
               long requireId =
                   requirePreAllocation(
                       request.getAppId(),
+                      shuffleId,
+                      partitionIds,
                       allocateSize,
                       request.getRetryMax(),
                       request.getRetryIntervalMax());
