@@ -42,7 +42,7 @@ public class ShuffleServerConf extends RssBaseConf {
           .doubleType()
           .defaultValue(0.6)
           .withDescription(
-              "JVM heap size * ratio for the maximum memory of buffer manager for shuffle server, this "
+              "JVM heap size or off-heap size(when enabling Netty) * ratio for the maximum memory of buffer manager for shuffle server, this "
                   + "is only effective when `rss.server.buffer.capacity` is not explicitly set");
 
   public static final ConfigOption<Long> SERVER_READ_BUFFER_CAPACITY =
@@ -56,7 +56,7 @@ public class ShuffleServerConf extends RssBaseConf {
           .doubleType()
           .defaultValue(0.2)
           .withDescription(
-              "JVM heap size * ratio for read buffer size, this is only effective when "
+              "JVM heap size or off-heap size(when enabling Netty) * ratio for read buffer size, this is only effective when "
                   + "`rss.server.reader.buffer.capacity.ratio` is not explicitly set");
 
   public static final ConfigOption<Long> SERVER_HEARTBEAT_DELAY =
@@ -77,6 +77,18 @@ public class ShuffleServerConf extends RssBaseConf {
           .defaultValue(10 * 1000L)
           .withDescription("Heartbeat interval to Coordinator (ms)");
 
+  public static final ConfigOption<Long> SERVER_NETTY_DIRECT_MEMORY_USAGE_TRACKER_DELAY =
+      ConfigOptions.key("rss.server.netty.directMemoryTracker.memoryUsage.initialFetchDelayMs")
+          .longType()
+          .defaultValue(10 * 1000L)
+          .withDescription("Direct memory usage tracker initial delay (ms)");
+
+  public static final ConfigOption<Long> SERVER_NETTY_DIRECT_MEMORY_USAGE_TRACKER_INTERVAL =
+      ConfigOptions.key("rss.server.netty.directMemoryTracker.memoryUsage.updateMetricsIntervalMs")
+          .longType()
+          .defaultValue(10 * 1000L)
+          .withDescription("Direct memory usage tracker interval to MetricSystem (ms)");
+
   public static final ConfigOption<Integer> SERVER_FLUSH_LOCALFILE_THREAD_POOL_SIZE =
       ConfigOptions.key("rss.server.flush.localfile.threadPool.size")
           .intType()
@@ -120,12 +132,6 @@ public class ShuffleServerConf extends RssBaseConf {
           .withDescription(
               "Expired time (ms) for application which has no heartbeat with coordinator");
 
-  public static final ConfigOption<Integer> SERVER_MEMORY_REQUEST_RETRY_MAX =
-      ConfigOptions.key("rss.server.memory.request.retry.max")
-          .intType()
-          .defaultValue(50)
-          .withDescription("Max times to retry for memory request");
-
   public static final ConfigOption<Long> SERVER_PRE_ALLOCATION_EXPIRED =
       ConfigOptions.key("rss.server.preAllocation.expired")
           .longType()
@@ -189,6 +195,16 @@ public class ShuffleServerConf extends RssBaseConf {
           .defaultValue(85.0)
           .withDescription("If disk usage is smaller than this value, disk can been written again");
 
+  public static final ConfigOption<Boolean> DISK_CAPACITY_WATERMARK_CHECK_ENABLED =
+      ConfigOptions.key("rss.server.disk-capacity.watermark.check.enabled")
+          .booleanType()
+          .defaultValue(false)
+          .withDescription(
+              "If it is co-located with other services, the high-low watermark check "
+                  + "based on the uniffle used is not correct. Due to this, the whole disk capacity "
+                  + "watermark check is necessary, which will reuse the current watermark value. "
+                  + "It will be disabled by default.");
+
   public static final ConfigOption<Long> PENDING_EVENT_TIMEOUT_SEC =
       ConfigOptions.key("rss.server.pending.event.timeout.sec")
           .longType()
@@ -271,6 +287,26 @@ public class ShuffleServerConf extends RssBaseConf {
           .noDefaultValue()
           .withDescription("The list of the Checker's name");
 
+  public static final ConfigOption<String> HEALTH_CHECKER_SCRIPT_PATH =
+      ConfigOptions.key("rss.server.health.checker.script.path")
+          .stringType()
+          .noDefaultValue()
+          .withDescription(
+              "The health script file path for HealthScriptChecker, if script file should have execute permission.");
+
+  public static final ConfigOption<Long> HEALTH_CHECKER_SCRIPT_EXECUTE_TIMEOUT =
+      ConfigOptions.key("rss.server.health.checker.script.execute.timeout")
+          .longType()
+          .defaultValue(5000L)
+          .withDescription("The health script file execute timeout ms.");
+
+  public static final ConfigOption<Long> HEALTH_CHECKER_LOCAL_STORAGE_EXECUTE_TIMEOUT =
+      ConfigOptions.key("rss.server.health.checker.localStorageExecutionTimeoutMS")
+          .longType()
+          .defaultValue(1000 * 60L)
+          .withDescription(
+              "The health checker for LocalStorageChecker execution timeout (Unit: ms). Default value is 1min");
+
   public static final ConfigOption<Double> SERVER_MEMORY_SHUFFLE_LOWWATERMARK_PERCENTAGE =
       ConfigOptions.key("rss.server.memory.shuffle.lowWaterMark.percentage")
           .doubleType()
@@ -296,29 +332,30 @@ public class ShuffleServerConf extends RssBaseConf {
               ConfigUtils.POSITIVE_LONG_VALIDATOR, "flush cold storage threshold must be positive")
           .defaultValue(64L * 1024L * 1024L)
           .withDescription(
-              "For multistorage, the event size exceed this value, flush data  to cold storage");
+              "For hybrid storage, the event size exceed this value, flush data  to cold storage");
 
-  public static final ConfigOption<String> MULTISTORAGE_MANAGER_SELECTOR_CLASS =
-      ConfigOptions.key("rss.server.multistorage.manager.selector.class")
+  public static final ConfigOption<String> HYBRID_STORAGE_MANAGER_SELECTOR_CLASS =
+      ConfigOptions.key("rss.server.hybrid.storage.manager.selector.class")
           .stringType()
-          .defaultValue("org.apache.uniffle.server.storage.multi.DefaultStorageManagerSelector")
+          .defaultValue("org.apache.uniffle.server.storage.hybrid.DefaultStorageManagerSelector")
           .withDescription(
-              "For multistorage, the storage manager selector strategy to support "
-                  + "policies of flushing to different storages");
+              "For hybrid storage, the storage manager selector strategy to support "
+                  + "policies of flushing to different storages")
+          .withDeprecatedKeys("rss.server.multistorage.manager.selector.class");
 
-  public static final ConfigOption<String> MULTISTORAGE_FALLBACK_STRATEGY_CLASS =
-      ConfigOptions.key("rss.server.multistorage.fallback.strategy.class")
+  public static final ConfigOption<String> HYBRID_STORAGE_FALLBACK_STRATEGY_CLASS =
+      ConfigOptions.key("rss.server.hybrid.storage.fallback.strategy.class")
           .stringType()
           .noDefaultValue()
-          .withDescription("For multistorage, fallback strategy class");
+          .withDescription("For hybrid storage, fallback strategy class")
+          .withDeprecatedKeys("rss.server.multistorage.fallback.strategy.class");
 
   public static final ConfigOption<Long> FALLBACK_MAX_FAIL_TIMES =
-      ConfigOptions.key("rss.server.multistorage.fallback.max.fail.times")
+      ConfigOptions.key("rss.server.hybrid.storage.fallback.max.fail.times")
           .longType()
-          .checkValue(
-              ConfigUtils.NON_NEGATIVE_LONG_VALIDATOR, " fallback times must be non-negative")
           .defaultValue(0L)
-          .withDescription("For multistorage, fail times exceed the number, will switch storage");
+          .withDescription("For hybrid storage, fail times exceed the number, will switch storage")
+          .withDeprecatedKeys("rss.server.multistorage.fallback.max.fail.times");
 
   public static final ConfigOption<List<String>> TAGS =
       ConfigOptions.key("rss.server.tags")
@@ -349,12 +386,6 @@ public class ShuffleServerConf extends RssBaseConf {
           .defaultValue(128 * 1024 * 1024L)
           .withDescription("The threshold of single shuffle buffer flush");
 
-  public static final ConfigOption<Long> STORAGEMANAGER_CACHE_TIMEOUT =
-      ConfigOptions.key("rss.server.multistorage.storagemanager.cache.timeout")
-          .longType()
-          .defaultValue(60 * 1000L)
-          .withDescription("The timeout of the cache which record the mapping information");
-
   public static final ConfigOption<Long> SERVER_LEAK_SHUFFLE_DATA_CHECK_INTERVAL =
       ConfigOptions.key("rss.server.leak.shuffledata.check.interval")
           .longType()
@@ -456,8 +487,11 @@ public class ShuffleServerConf extends RssBaseConf {
   public static final ConfigOption<Integer> NETTY_SERVER_WORKER_THREAD =
       ConfigOptions.key("rss.server.netty.worker.thread")
           .intType()
-          .defaultValue(100)
-          .withDescription("Worker thread count in netty");
+          .defaultValue(0)
+          .withDescription(
+              "Worker thread count in netty. When set to 0, "
+                  + "the default value is dynamically set to twice the number of processor cores, "
+                  + "but it will not be less than 100 to ensure the minimum throughput of the service.");
 
   public static final ConfigOption<Long> SERVER_NETTY_HANDLER_IDLE_TIMEOUT =
       ConfigOptions.key("rss.server.netty.handler.idle.timeout")
@@ -499,6 +533,45 @@ public class ShuffleServerConf extends RssBaseConf {
                   + "network_bandwidth = 10Gbps, buffer size should be ~ 1.25MB."
                   + "Default is 0, OS will dynamically adjust the buf size.");
 
+  public static final ConfigOption<Integer> TOP_N_APP_SHUFFLE_DATA_SIZE_NUMBER =
+      ConfigOptions.key("rss.server.topN.appShuffleDataSize.number")
+          .intType()
+          .defaultValue(10)
+          .withDescription("number of topN shuffle data size of app level.");
+
+  public static final ConfigOption<Integer> TOP_N_APP_SHUFFLE_DATA_REFRESH_INTERVAL =
+      ConfigOptions.key("rss.server.topN.appShuffleDataSize.refreshIntervalMs")
+          .intType()
+          .defaultValue(1000)
+          .withDescription(
+              "refresh interval in ms for TopN shuffle data size of app level calc task.");
+
+  public static final ConfigOption<Integer> SUMMARY_METRIC_WAIT_QUEUE_SIZE =
+      ConfigOptions.key("rss.server.summary.metric.wait.queue.size")
+          .intType()
+          .defaultValue(1000)
+          .withDescription(
+              "size of waiting queue for thread pool that used for calc summary metric.");
+
+  public static final ConfigOption<Integer> SUMMARY_METRIC_THREAD_POOL_CORE_SIZE =
+      ConfigOptions.key("rss.server.summary.metric.thread.pool.core.size")
+          .intType()
+          .defaultValue(2)
+          .withDescription("core thread number of thread pool that used for calc summary metric.");
+
+  public static final ConfigOption<Integer> SUMMARY_METRIC_THREAD_POOL_MAX_SIZE =
+      ConfigOptions.key("rss.server.summary.metric.thread.pool.max.size")
+          .intType()
+          .defaultValue(20)
+          .withDescription("max thread number of thread pool that used for calc summary metric.");
+
+  public static final ConfigOption<Integer> SUMMARY_METRIC_THREAD_POOL_KEEP_ALIVE_TIME =
+      ConfigOptions.key("rss.server.summary.metric.thread.pool.keep.alive.time")
+          .intType()
+          .defaultValue(60)
+          .withDescription(
+              "keep alive time of thread pool that used for calc summary metric, in SECONDS.");
+
   public ShuffleServerConf() {}
 
   public ShuffleServerConf(String fileName) {
