@@ -18,12 +18,13 @@
 package org.apache.uniffle.server.buffer;
 
 public class PreAllocatedBufferInfo {
-
+  private String appId;
   private long requireId;
   private long timestamp;
   private int requireSize;
 
-  public PreAllocatedBufferInfo(long requireId, long timestamp, int requireSize) {
+  public PreAllocatedBufferInfo(String appId, long requireId, long timestamp, int requireSize) {
+    this.appId = appId;
     this.requireId = requireId;
     this.timestamp = timestamp;
     this.requireSize = requireSize;
@@ -41,4 +42,7 @@ public class PreAllocatedBufferInfo {
     return requireSize;
   }
 
+  public String getAppId() {
+    return appId;
+  }
 }
