@@ -34,7 +34,6 @@ import org.apache.uniffle.storage.common.Storage;
 import org.apache.uniffle.storage.common.StorageWriteMetrics;
 import org.apache.uniffle.storage.handler.api.ShuffleWriteHandler;
 
-
 public abstract class SingleStorageManager implements StorageManager {
 
   private static final Logger LOG = LoggerFactory.getLogger(SingleStorageManager.class);
@@ -55,15 +54,6 @@ public abstract class SingleStorageManager implements StorageManager {
   public boolean write(Storage storage, ShuffleWriteHandler handler, ShuffleDataFlushEvent event) {
     String shuffleKey = RssUtils.generateShuffleKey(event.getAppId(), event.getShuffleId());
     storage.createMetadataIfNotExist(shuffleKey);
-
-    boolean locked = storage.lockShuffleShared(shuffleKey);
-    if (!locked) {
-      LOG.warn("AppId {} shuffleId {} was removed already, lock don't exist {} should be dropped,"
-          + " may leak one handler", event.getAppId(), event.getShuffleId(), event);
-      throw new IllegalStateException("AppId " + event.getAppId() + " ShuffleId " + event.getShuffleId()
-        + " was removed");
-    }
-
     try {
       long startWrite = System.currentTimeMillis();
       handler.write(event.getShuffleBlocks());
@@ -74,8 +64,6 @@ public abstract class SingleStorageManager implements StorageManager {
       LOG.warn("Exception happened when write data for " + event + ", try again", e);
       ShuffleServerMetrics.counterWriteException.inc();
       Uninterruptibles.sleepUninterruptibly(1000, TimeUnit.MILLISECONDS);
-    } finally {
-      storage.unlockShuffleShared(shuffleKey);
     }
     return false;
   }
@@ -113,7 +101,6 @@ public abstract class SingleStorageManager implements StorageManager {
     }
   }
 
-
   @Override
   public boolean canWrite(ShuffleDataFlushEvent event) {
     try {
@@ -128,8 +115,9 @@ public abstract class SingleStorageManager implements StorageManager {
       return false;
     }
   }
-  
-  public StorageWriteMetrics createStorageWriteMetrics(ShuffleDataFlushEvent event, long writeTime) {
+
+  public StorageWriteMetrics createStorageWriteMetrics(
+      ShuffleDataFlushEvent event, long writeTime) {
     long length = 0;
     long blockNum = 0;
     for (ShufflePartitionedBlock block : event.getShuffleBlocks()) {
@@ -137,7 +125,9 @@ public abstract class SingleStorageManager implements StorageManager {
       blockNum++;
     }
     List<Integer> partitions = Lists.newArrayList();
-    for (int partition = event.getStartPartition(); partition <= event.getEndPartition(); partition++) {
+    for (int partition = event.getStartPartition();
+        partition <= event.getEndPartition();
+        partition++) {
       partitions.add(partition);
     }
     return new StorageWriteMetrics(
